@@ -2,6 +2,7 @@ from functools import lru_cache
 
 import gym
 import numpy as np
+import torch
 from sample_factory.envs.env_registry import global_env_registry
 from transformers import RobertaTokenizerFast
 
@@ -16,6 +17,9 @@ class SampleFactoryNLELanguageEnv(gym.Env):
 
         self.cfg = cfg
         self.observation_space = gym.spaces.Dict()
+        self.observation_space.spaces["obs"] = gym.spaces.Box(
+            0, 1000000, shape=(1,), dtype=np.int32
+        )
         self.observation_space.spaces["input_ids"] = gym.spaces.Box(
             0, 1000000, shape=(self.cfg["max_token_length"],), dtype=np.int32
         )
@@ -26,7 +30,7 @@ class SampleFactoryNLELanguageEnv(gym.Env):
         self.env = NLELanguageWrapper(self.nle_env, use_language_action=False)
         self.action_space = self.env.action_space
         self.tokenizer = RobertaTokenizerFast.from_pretrained(
-            "distilroberta-base", truncation_side="left", local_files_only=True
+            "distilroberta-base", truncation_side="left"
         )
 
     # We use caching to avoid re-tokenizing observations that are already seen.
@@ -39,6 +43,8 @@ class SampleFactoryNLELanguageEnv(gym.Env):
             truncation=True,
             max_length=self.cfg["max_token_length"],
         )
+        # Sample factory insists on normalizing obs key.
+        tokens.data["obs"] = torch.zeros(1)
         return tokens.data
 
     def _convert_obsv_to_str(self, obsv):
