@@ -246,15 +246,15 @@ class NLELanguageWrapper(Wrapper):
         # Build map for action string to NLE Action Enum
         self.action_str_enum_map = {}
         for nle_action_enum, action_strs in self.all_nle_action_map.items():
-            if nle_action_enum in self.env._actions:
+            if nle_action_enum in self.env.actions:
                 for action_str in action_strs:
                     self.action_str_enum_map[action_str] = nle_action_enum
 
         # Build map for NLE Action Enum to NLE action index
         self.action_enum_index_map = {}
         for nle_action_enum, _ in self.all_nle_action_map.items():
-            if nle_action_enum in self.env._actions:
-                self.action_enum_index_map[nle_action_enum] = self.env._actions.index(
+            if nle_action_enum in self.env.actions:
+                self.action_enum_index_map[nle_action_enum] = self.env.actions.index(
                     nle_action_enum
                 )
 
