@@ -16,6 +16,7 @@
 #include <memory>
 #include <set>
 #include <string>
+#include <unordered_map>
 #include <vector>
 
 #include "include/display.h"
@@ -60,6 +61,12 @@ class NLELanguageObsv {
   py::bytes text_message(py::array_t<uint8_t> tty_chars);
 
  private:
+  std::unordered_map<int64_t, std::string> alignment_map{
+      {A_NONE, "None"},
+      {A_LAWFUL, "Lawful"},
+      {A_NEUTRAL, "Neutral"},
+      {A_CHAOTIC, "Chaotic"}};
+
   const std::set<std::string> cmap_floor{
       "room floor",
       "dark room floor",
@@ -205,7 +212,33 @@ class NLELanguageObsv {
       "explosion bottom right",
       "MAXPCHARS",
   };
-  std::string Im_unused;
+
+  std::unordered_map<int64_t, std::string> condition_map = {
+      {BL_MASK_STONE, "Stoned"},
+      {BL_MASK_SLIME, "Slimed"},
+      {BL_MASK_STRNGL, "Strangled"},
+      {BL_MASK_FOODPOIS, "Food Poisoning"},
+      {BL_MASK_TERMILL, "Terminally Ill"},
+      {BL_MASK_BLIND, "Blind"},
+      {BL_MASK_DEAF, "Deaf"},
+      {BL_MASK_STUN, "Stunned"},
+      {BL_MASK_CONF, "Confused"},
+      {BL_MASK_HALLU, "Hallucinating"},
+      {BL_MASK_LEV, "Levitating"},
+      {BL_MASK_FLY, "Flying"},
+      {BL_MASK_RIDE, "Riding"},
+  };
+
+  std::unordered_map<int64_t, std::string> encumbrance_map{
+      {UNENCUMBERED, "Unencumbered"}, {SLT_ENCUMBER, "Burdened"},
+      {MOD_ENCUMBER, "Stressed"},     {HVY_ENCUMBER, "Strained"},
+      {EXT_ENCUMBER, "Overtaxed"},    {OVERLOADED, "Overloaded"}};
+
+  std::unordered_map<int64_t, std::string> hunger_map{
+      {SATIATED, "Satiated"}, {NOT_HUNGRY, "Not Hungry"}, {HUNGRY, "Hungry"},
+      {WEAK, "Weak"},         {FAINTING, "Fainting"},     {FAINTED, "Fainted"},
+      {STARVED, "Starved"}};
+
   void build_fullscreen_view_glyph_map();
   std::list<std::tuple<std::string, std::string, std::string>>
   sort_by_distance_direction(
@@ -233,7 +266,7 @@ class NLELanguageObsv {
 
   std::array<std::string, MAX_GLYPH> fullscreen_view_glyph_map;
   std::array<std::string, MAX_GLYPH> visual_view_glyph_map;
-  std::map<std::string, std::string> noun_plural_lookup;
+  std::unordered_map<std::string, std::string> noun_plural_lookup;
   std::array<
       std::array<std::pair<std::string, std::string>, DUNGEON_HEIGHT * 2>,
       DUNGEON_WIDTH * 2>
@@ -335,11 +368,11 @@ NLELanguageObsv::compress_by_glyph(
         glyph_distance_direction) {
   std::list<std::tuple<std::string, std::string, std::string>>
       new_glyph_distance_direction;
-  std::vector<std::pair<std::string, std::string>>
+  std::list<std::pair<std::string, std::string>>
       vector_glyh_distance_to_directions;
-  std::vector<std::pair<std::string, std::string>>
+  std::list<std::pair<std::string, std::string>>
       new_vector_glyh_distance_to_directions;
-  std::map<std::pair<std::string, std::string>, std::vector<std::string>>
+  std::map<std::pair<std::string, std::string>, std::list<std::string>>
       map_glyh_distance_to_directions;
 
   for (auto it = glyph_distance_direction.begin();
@@ -349,7 +382,7 @@ NLELanguageObsv::compress_by_glyph(
     std::string direction_string = std::get<2>(*it);
 
     std::pair<std::string, std::string> glyph_distance_string_pair =
-        std::make_pair(glyph_string, distance_string);
+        std::pair{glyph_string, distance_string};
     // We use the vector to retain the sorted order.
     if (map_glyh_distance_to_directions.find(glyph_distance_string_pair) ==
         map_glyh_distance_to_directions.end()) {
@@ -358,19 +391,19 @@ NLELanguageObsv::compress_by_glyph(
     map_glyh_distance_to_directions[glyph_distance_string_pair].push_back(
         direction_string);
   }
-  std::map<std::pair<std::string, std::string>, std::vector<std::string>>
+  std::map<std::pair<std::string, std::string>, std::list<std::string>>
       new_map_glyh_distance_to_directions;
 
   for (auto it = vector_glyh_distance_to_directions.begin();
        it != vector_glyh_distance_to_directions.end(); it++) {
     std::string glyph_string = it->first;
     std::string distance_string = it->second;
-    std::vector<std::string> direction_strings =
+    std::list<std::string> direction_strings =
         map_glyh_distance_to_directions[*it];
-    std::map<std::string, int> direction_freq;
-    std::vector<std::string> direction_freq_order;
-    std::vector<std::string> multiple_in_direction;
-    std::vector<std::string> single_direction_strings;
+    std::unordered_map<std::string, int> direction_freq;
+    std::list<std::string> direction_freq_order;
+    std::list<std::string> multiple_in_direction;
+    std::list<std::string> single_direction_strings;
 
     // For the current glyph & distance check the frequency of each direction.
     for (auto direction_it = direction_strings.begin();
@@ -397,7 +430,7 @@ NLELanguageObsv::compress_by_glyph(
     }
     if (multiple_in_direction.size() > 0) {
       std::pair<std::string, std::string> glyph_distance_string_pair =
-          std::make_pair(noun_plural_lookup[glyph_string], distance_string);
+          std::pair{noun_plural_lookup[glyph_string], distance_string};
       new_vector_glyh_distance_to_directions.push_back(
           glyph_distance_string_pair);
       new_map_glyh_distance_to_directions[glyph_distance_string_pair] =
@@ -411,26 +444,28 @@ NLELanguageObsv::compress_by_glyph(
        it != vector_glyh_distance_to_directions.end(); ++it) {
     std::string glyph_string = it->first;
     std::string distance_string = it->second;
-    std::vector<std::string> direction_strings =
+    std::list<std::string> direction_strings =
         map_glyh_distance_to_directions[*it];
 
     if (direction_strings.size() == 1) {
-      std::string first_direction = direction_strings[0];
+      std::string first_direction = direction_strings.front();
       new_glyph_distance_direction.push_back(
           make_tuple(glyph_string, distance_string, first_direction));
     } else if (direction_strings.size() == 2) {
-      std::string first_direction = direction_strings[0];
-      std::string second_direction = direction_strings[1];
+      std::string first_direction = direction_strings.front();
+      std::string second_direction = *std::next(direction_strings.begin());
       std::string new_direction = first_direction + " and " + second_direction;
       new_glyph_distance_direction.push_back(
           make_tuple(glyph_string, distance_string, new_direction));
     } else {
-      std::string last_direction =
-          direction_strings[direction_strings.size() - 1];
-      std::string new_direction = "";
-      new_direction += direction_strings[0];
-      for (uint64_t i = 1; i < direction_strings.size() - 1; i++) {
-        new_direction += ", " + direction_strings[i];
+      std::string last_direction = direction_strings.back();
+      std::string new_direction = direction_strings.front();
+      direction_strings.pop_back();
+      direction_strings.pop_front();
+      for (auto direction_strings_it = direction_strings.begin();
+           direction_strings_it != direction_strings.end();
+           ++direction_strings_it) {
+        new_direction += ", " + *direction_strings_it;
       }
       new_direction += ", and " + last_direction;
       new_glyph_distance_direction.push_back(
@@ -451,7 +486,7 @@ std::pair<int, int> NLELanguageObsv::local_glyph_to_global(int glyph_idx,
   int glyph_relative_start_0_x = glyph_relative_x + DUNGEON_WIDTH;
   int glyph_relative_start_0_y = glyph_relative_y + DUNGEON_HEIGHT;
 
-  return std::make_pair(glyph_relative_start_0_x, glyph_relative_start_0_y);
+  return std::pair{glyph_relative_start_0_x, glyph_relative_start_0_y};
 }
 
 std::list<std::tuple<std::string, std::string, std::string>>
@@ -965,7 +1000,7 @@ std::pair<std::string, std::string> NLELanguageObsv::pos_to_str(int x, int y) {
   std::string distance_str = str_offset;
   std::string direction_str =
       position_descriptor_y + position_descriptor_x + diagonal;
-  return std::make_pair(distance_str, direction_str);
+  return std::pair{distance_str, direction_str};
 }
 
 std::string NLELanguageObsv::offset_to_str(int offset) {
@@ -992,124 +1027,70 @@ py::bytes NLELanguageObsv::text_blstats(py::array_t<int64_t> blstats) {
   py::buffer_info blstats_buffer = blstats.request();
   int64_t *blstats_data = reinterpret_cast<int64_t *>(blstats_buffer.ptr);
 
-  std::string alignment_str;
-  switch (blstats_data[26]) {
-    case A_NONE:
-      alignment_str = "None";
-      break;
-    case A_LAWFUL:
-      alignment_str = "Lawful";
-      break;
-    case A_NEUTRAL:
-      alignment_str = "Neutral";
-      break;
-    case A_CHAOTIC:
-      alignment_str = "Chaotic";
-      break;
-  }
+  std::string alignment_str = alignment_map[blstats_data[26]];
+  std::string hunger_str = hunger_map[blstats_data[21]];
+  std::string encumbrance_str = encumbrance_map[blstats_data[22]];
 
-  std::string hunger_str;
-  switch (blstats_data[21]) {
-    case SATIATED:
-      hunger_str = "Satiated";
-      break;
-    case NOT_HUNGRY:
-      hunger_str = "Not Hungry";
-      break;
-    case HUNGRY:
-      hunger_str = "Hungry";
-      break;
-    case WEAK:
-      hunger_str = "Weak";
-      break;
-    case FAINTING:
-      hunger_str = "Fainting";
-      break;
-    case FAINTED:
-      hunger_str = "Fainted";
-      break;
-    case STARVED:
-      hunger_str = "Starved";
-      break;
-  }
+  std::unordered_map<int64_t, std::string> condition_map = {
+      {BL_MASK_STONE, "Stoned"},
+      {BL_MASK_SLIME, "Slimed"},
+      {BL_MASK_STRNGL, "Strangled"},
+      {BL_MASK_FOODPOIS, "Food Poisoning"},
+      {BL_MASK_TERMILL, "Terminally Ill"},
+      {BL_MASK_BLIND, "Blind"},
+      {BL_MASK_DEAF, "Deaf"},
+      {BL_MASK_STUN, "Stunned"},
+      {BL_MASK_CONF, "Confused"},
+      {BL_MASK_HALLU, "Hallucinating"},
+      {BL_MASK_LEV, "Levitating"},
+      {BL_MASK_FLY, "Flying"},
+      {BL_MASK_RIDE, "Riding"},
+  };
 
-  std::string encumberance_str;
-  switch (blstats_data[22]) {
-    case UNENCUMBERED:
-      encumberance_str = "Unemcumbered";
-      break;
-    case SLT_ENCUMBER:
-      encumberance_str = "Burdened";
-      break;
-    case MOD_ENCUMBER:
-      encumberance_str = "Stressed";
-      break;
-    case HVY_ENCUMBER:
-      encumberance_str = "Strained";
-      break;
-    case EXT_ENCUMBER:
-      encumberance_str = "Oveexrtaxed";
-      break;
-    case OVERLOADED:
-      encumberance_str = "Overloaded";
-      break;
+  std::vector<std::string> conditions;
+  for (const auto &[mask, condition] : condition_map) {
+    if (blstats_data[25] & mask) {
+      conditions.push_back(condition);
+    }
   }
 
   std::string condition;
-  std::vector<std::string> conditions;
-  if (blstats_data[25] & BL_MASK_STONE) conditions.push_back("Stoned");
-  if (blstats_data[25] & BL_MASK_SLIME) conditions.push_back("Slimed");
-  if (blstats_data[25] & BL_MASK_STRNGL) conditions.push_back("Strangled");
-  if (blstats_data[25] & BL_MASK_FOODPOIS)
-    conditions.push_back("Food Poisoning");
-  if (blstats_data[25] & BL_MASK_TERMILL)
-    conditions.push_back("Terminally Ill");
-  if (blstats_data[25] & BL_MASK_BLIND) conditions.push_back("Blind");
-  if (blstats_data[25] & BL_MASK_DEAF) conditions.push_back("Deaf");
-  if (blstats_data[25] & BL_MASK_STUN) conditions.push_back("Stunned");
-  if (blstats_data[25] & BL_MASK_CONF) conditions.push_back("Confused");
-  if (blstats_data[25] & BL_MASK_HALLU) conditions.push_back("Hallucinating");
-  if (blstats_data[25] & BL_MASK_LEV) conditions.push_back("Levitating");
-  if (blstats_data[25] & BL_MASK_FLY) conditions.push_back("Flying");
-  if (blstats_data[25] & BL_MASK_RIDE) conditions.push_back("Riding");
-  if (conditions.size() == 0)
+  if (conditions.empty()) {
     condition = "None";
-  else if (conditions.size() == 1)
+  } else if (conditions.size() == 1) {
     condition = conditions[0];
-  else if (conditions.size() > 1) {
+  } else {
     condition = conditions[0];
     for (auto it = ++conditions.begin(); it != conditions.end(); ++it) {
       condition += " " + *it;
     }
   }
 
-  std::string output =
-      ("Strength: " + std::to_string(blstats_data[3]) + "/" +
-       std::to_string(blstats_data[2]) + "\n" +
-       "Dexterity: " + std::to_string(blstats_data[4]) + "\n" +
-       "Constitution: " + std::to_string(blstats_data[5]) + "\n" +
-       "Intelligence: " + std::to_string(blstats_data[6]) + "\n" +
-       "Wisdom: " + std::to_string(blstats_data[7]) + "\n" +
-       "Charisma: " + std::to_string(blstats_data[8]) + "\n" +
-       "Depth: " + std::to_string(blstats_data[12]) + "\n" +
-       "Gold: " + std::to_string(blstats_data[13]) + "\n" +
-       "HP: " + std::to_string(blstats_data[10]) + "/" +
-       std::to_string(blstats_data[11]) + "\n" +
-       "Energy: " + std::to_string(blstats_data[14]) + "/" +
-       std::to_string(blstats_data[15]) + "\n" +
-       "AC: " + std::to_string(blstats_data[16]) + "\n" +
-       "XP: " + std::to_string(blstats_data[18]) + "/" +
-       std::to_string(blstats_data[19]) + "\n" +
-       "Time: " + std::to_string(blstats_data[20]) + "\n" +
-       "Position: " + std::to_string(blstats_data[0]) + "|" +
-       std::to_string(blstats_data[1]) + "\n" + "Hunger: " + hunger_str + "\n" +
-       "Monster Level: " + std::to_string(blstats_data[17]) + "\n" +
-       "Encumbrance: " + encumberance_str + "\n" +
-       "Dungeon Number: " + std::to_string(blstats_data[23]) + "\n" +
-       "Level Number: " + std::to_string(blstats_data[24]) + "\n" +
-       "Score: " + std::to_string(blstats_data[9]) + "\n" +
-       "Alignment: " + alignment_str + "\n" + "Condition: " + condition);
-  return py::bytes(output);
+  std::stringstream ss;
+  ss << "Strength: " << blstats_data[3] << "/" << blstats_data[2] << "\n"
+     << "Dexterity: " << blstats_data[4] << "\n"
+     << "Constitution: " << blstats_data[5] << "\n"
+     << "Intelligence: " << blstats_data[6] << "\n"
+     << "Wisdom: " << blstats_data[7] << "\n"
+     << "Charisma: " << blstats_data[8] << "\n"
+     << "Depth: " << blstats_data[12] << "\n"
+     << "Gold: " << blstats_data[13] << "\n"
+     << "HP: " << blstats_data[10] << "/" << blstats_data[11] << "\n"
+     << "Energy: " << blstats_data[14] << "/" << blstats_data[15] << "\n"
+     << "AC: " << blstats_data[16] << "\n"
+     << "XP: " << blstats_data[18] << "/" << blstats_data[19] << "\n"
+     << "Time: " << blstats_data[20] << "\n"
+     << "Position: " << blstats_data[0] << "|" << blstats_data[1] << "\n"
+     << "Hunger: " << hunger_str << "\n"
+     << "Monster Level: " << blstats_data[17] << "\n"
+     << "Encumbrance: " << encumbrance_str << "\n"
+     << "Dungeon Number: " << blstats_data[23] << "\n"
+     << "Level Number: " << blstats_data[24] << "\n"
+     << "Score: " << blstats_data[9] << "\n"
+     << "Alignment: " << alignment_str << "\n"
+     << "Condition: " << condition;
+
+  return py::bytes(ss.str());
 }
 
 std::string NLELanguageObsv::trim(std::string input) {
